package example;

/*
 * DemoClik.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.URISyntaxException;

import javax.robotics.engine.clik.CLIK;
import javax.robotics.engine.clik.ClikJacobianInverse;
import javax.robotics.engine.clik.ClikJacobianTranspose;
import javax.robotics.engine.robots.DHRobot3D;
import javax.robotics.engine.robots.Robot3D;
import javax.robotics.engine.trajectory.Spline;
import javax.robotics.plot.MultiPlotPanel;
import javax.robotics.simulator.CLIKSimulation;
import javax.robotics.swing.LookAndFeelSelector;
import javax.robotics.swing.RobotPanel;
import javax.robotics.vecmath.DMatrix;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.apache.xmlbeans.XmlException;
/**
 * CLIK simulator demonstration. The application shows how to create a CLIK
 * simulation.
 * 
 * @author Carmine Lia
 * 
 */
public class DemoClik implements ActionListener
{

    private static final long serialVersionUID = 3256725099943049012L;

    // robotModel
    final Robot3D robot;
    final CLIK clik;
    final DMatrix Kp, Kd;
    final Spline trj;
    final CLIKSimulation clikSim;
    final JFrame frame; 

    // parametri per la simulazione
    static final float dt = 0.001f;
    static final float dur = 6.0f;
    private final int punti = Math.round(dur / dt) + 1;

    private DemoClik() throws XmlException, IOException, URISyntaxException
    {

        try
        {
            LookAndFeelSelector.setSkin(getClass().getResourceAsStream(
                    "/example/theme/themepack.zip"));
        }
        catch (SecurityException e)
        {
            e.printStackTrace();
        }
        catch (IllegalArgumentException e)
        {
            e.printStackTrace();
        }
        catch (NoSuchMethodException e)
        {
            e.printStackTrace();
        }
        catch (IllegalAccessException e)
        {
            e.printStackTrace();
        }
        catch (InvocationTargetException e)
        {
            e.printStackTrace();
        }

        frame = new JFrame();
        
        Kp = new DMatrix(200, 200, 200);
        Kd = new DMatrix(100, 100, 100);

        robot = new DHRobot3D(getClass().getResourceAsStream(
                "/example/conf/puma560_nodynamics.xml"));

        // create the end effector path
        trj = new Spline(getClass().getResourceAsStream(
                "/example/conf/spline_path.xml"));

        String[] possibleValues =
            { "inverse", "transpose" };
        String selectedValue = (String) JOptionPane.showInputDialog(null,
                "CLIK jacobian", "Choose the CLIK",
                JOptionPane.INFORMATION_MESSAGE, null, possibleValues,
                possibleValues[0]);

        if(selectedValue == null)
            System.exit(0);

        if(selectedValue.equals("transpose"))
        {
            clik = new ClikJacobianTranspose(robot, Kp, Kd);
            frame.setTitle("PUMA560 CLIK with Jacobian transpose");
        }
        else
        {
            clik = new ClikJacobianInverse(robot, Kp, Kd);
            frame.setTitle("PUMA560 CLIK with Jacobian inverse");
        }

        clik.setEps(0.01);

        // create the simulator
        clikSim = new CLIKSimulation(robot, clik, trj, punti);

        // get the plots panel of simulator
        final MultiPlotPanel plotsPanel = clikSim.getPlotPanel();
        plotsPanel.setLayout(new GridLayout(4, 1));

        // get the robot panel of simulator
        final RobotPanel robotPanel = clikSim.getRobotPanel();

        // the side panel with plots an play sim button
        final JPanel p = new JPanel();
        p.setLayout(new BorderLayout());

        final JButton b = new JButton("Play simulation");
        b.addActionListener(this);

        p.add(plotsPanel, BorderLayout.CENTER);
        p.add(b, BorderLayout.SOUTH);

        frame.setPreferredSize(new Dimension(1024, 768));
        frame.getContentPane().add(robotPanel, BorderLayout.CENTER);
        frame.getContentPane().add(p, BorderLayout.EAST);
        frame.pack();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setVisible(true);

        // start the simulator
        clikSim.setEnable(true);

    }

    public static void main(String[] args) throws XmlException, IOException,
            URISyntaxException
    {
        new DemoClik();
    }

    // button action
    public void actionPerformed(ActionEvent e)
    {
        // play the simulation
        clikSim.play();
    }
}