package example;

/*
 * DemoControlRRA.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URISyntaxException;

import javax.robotics.engine.controllers.EndEffController;
import javax.robotics.engine.controllers.ResolvedRateAcceleration;
import javax.robotics.engine.robots.DHRobot3D;
import javax.robotics.engine.robots.Robot3D;
import javax.robotics.engine.trajectory.Spline;
import javax.robotics.engine.trajectory.Trajectory;
import javax.robotics.ode.RungeKutta4th2;
import javax.robotics.ode.Solver2;
import javax.robotics.plot.MultiPlotPanel;
import javax.robotics.simulator.EndEffPosControlSimulation;
import javax.robotics.swing.RobotPanel;
import javax.robotics.vecmath.RVector;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.apache.xmlbeans.XmlException;
/**
 * End Effector control simulator demonstration. The Resolved Rate Acceleration
 * control demo.
 * 
 * @author Carmine Lia
 * 
 */
public class DemoControlRRA extends JFrame implements ActionListener
{
    /**
     * 
     */
    private static final long serialVersionUID = 3256725099943049012L;

    // robotModel
    private final Robot3D robot;
    // control
    private final EndEffController rra;
    // path
    private final Trajectory trj;
    // ode solver
    private final Solver2 solver;
    // control simulator
    private final EndEffPosControlSimulation sim;

    // parametri per la simulazione
    private final double dt = 0.001;
    private final int punti = 3000;

    final ClassLoader cl = this.getClass().getClassLoader();

    private DemoControlRRA(String title) throws XmlException, IOException,
            URISyntaxException
    {
        super(title);
        // create the robot
        robot = new DHRobot3D(cl
                .getResourceAsStream("example/conf/puma560.xml"));

        // create the end eff path
        trj = new Spline(cl
                .getResourceAsStream("example/conf/position_spline_path.xml"));

        // init solver with initial state
        solver = new RungeKutta4th2(dt, new RVector(6), new RVector(6));

        // create the controller
        rra = new ResolvedRateAcceleration(robot, 600.0, 2500.0, 0.0, 0.0);

        // create the simulator
        sim = new EndEffPosControlSimulation(robot, rra, solver, trj, punti);
        // get the plot panel
        MultiPlotPanel plotsPanel = sim.getPlotPanel();
        plotsPanel.setLayout(new GridLayout(4, 1));

        // get the robot panel
        RobotPanel robotPanel = sim.getRobotPanel();

        // side panel with plots and the play sim button
        final JPanel p = new JPanel();
        p.setLayout(new BorderLayout());

        final JButton b = new JButton("Play simulation");
        b.addActionListener(this);

        p.add(plotsPanel, BorderLayout.CENTER);
        p.add(b, BorderLayout.SOUTH);

        setPreferredSize(new Dimension(1024, 768));
        getContentPane().add(robotPanel, BorderLayout.CENTER);
        getContentPane().add(p, BorderLayout.EAST);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        pack();

        setVisible(true);

        // start the simulator
        sim.setEnable(true);
    }

    public void actionPerformed(ActionEvent e)
    {
        // play the simulation
        sim.play();
    }

    public static void main(String[] args) throws XmlException, IOException,
            URISyntaxException
    {
        final String s = "PUMA560 Resolved rate acceleration control";
        new DemoControlRRA(s);
    }

}