/*
 * DemoMatrix.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package example;

import javax.robotics.swing.DisplayPanel;
import javax.robotics.vecmath.Matrix;
import javax.robotics.vecmath.RVector;

/**
 * SVD and Lyapunov functions demo
 * 
 * @author Carmine Lia
 * 
 */
public class DemoMatrix
{
    /**
     * @param args
     */
    public static void main(String[] args)
    {
        final String newLine = System.getProperty("line.separator");

        final DisplayPanel out = new DisplayPanel("Matrix Demo Output");
        out.setDefaultCloseOperation(DisplayPanel.EXIT_ON_CLOSE);
        
        final double[] valR =
            { 1.4, 2.5, 3.6, 4.9, 3.0, 4.6, 9.9, 3.8, 5.7, 10.45, 8.04, 7.5 };
        final double[] valS =
            { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -3, 0.0, -5, 0.0, 0.0,
                    -4, 0.0, -6 };
        /*
         * double[] valRs = { 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, -3, 0.0,
         * -5, 0.0, 0.0, -4, 0.0, -6, -3, 4, -4, 8 };
         */
        final double[] valSs =
            { 0.0, 0.0, 1.0, 0.0, 0.0, -2, 0.0, 1.0, -3, 0.0, -5, 0.0, 3.02,
                    -4.12, 8.96, 1.992 };

        final Matrix M = new Matrix(4, 3, valR);
        final Matrix S = new Matrix(4, valS);
        final Matrix Ss = new Matrix(4, valSs);
        // Matrix Rs = new Matrix(5, 4, valRs);
        final int m = M.getNumRow();
        int n = M.getNumCol();

        final Matrix Q = new Matrix(n, n);
        final Matrix U = new Matrix(m, n);
        Matrix V = new Matrix(n, n);
        RVector s = new RVector(n);

        out.append("===============================" + newLine);
        out.append("      Rectangular matrix" + newLine);
        out.append("===============================" + newLine);
        out.append("R:" + newLine + M);

        out
                .append("----- Computes Singular Value Decomposition Jacobi method -----"
                        + newLine);
        int rank = M.svd(s);
        out.append("rank: " + rank);
        out.append("Singular values:" + newLine + s);
        out.append(String.format("Condition number:%9.4f" + newLine + newLine,
                s.getElement(0) / s.getElement(rank - 1)));

        out
                .append("---- Computes Singular Value Decomposition -----"
                        + newLine);
        rank = M.svd(Q, U, V);
        out.append("Q" + newLine + Q);
        out.append("U" + newLine + U);
        out.append("V" + newLine + V);
        Q.mulTransposeRight(Q, V);
        U.mul(Q);
        out.append("Verify U*Q*V'=R" + newLine + U);

        out
                .append("----- Computes left pseudo-inverse  matrix using QR decomposition -----"
                        + newLine);
        final Matrix PinvQR = new Matrix(M.inverse());
        final Matrix I = new Matrix(Math.min(m, n));
        out.append("Left pseudo Inverse:" + newLine + PinvQR);
        I.mul(PinvQR, M);
        out.append("computes pinv(R)*R" + newLine + I);

        out.append("----- Computes right pseudo-inverse matrix using SVD -----"
                + newLine);
        M.transpose();
        final Matrix Pinv = new Matrix(M.pseudoInverse());
        out.append("Right pseudo Inverse:" + newLine + Pinv);
        I.mul(M, Pinv);
        out.append("computes R*pinv(R)" + newLine + I);

        out.append("===============================" + newLine);
        out.append("       Square matrix" + newLine);
        out.append("===============================" + newLine);

        out.append("S:" + newLine + S);
        n = S.getNumCol();
        out
                .append("----- Computes inverse matrix using Java3D inner function -----"
                        + newLine);
        Matrix Si = new Matrix(n);
        Si.invert(S);
        out.append("Inverse:" + newLine + Si);

        out.append("----- Computes inverse matrix using LU decomposition -----"
                + newLine);
        Si = S.inverse();
        out.append("Inverse:" + newLine + Si);

        V = new Matrix(n, n);
        final Matrix D = new Matrix(n);
        RVector d = new RVector(n);
        final RVector e = new RVector(n);
        out.append("----- Computes Eigen value decomposition:" + newLine);
        S.eig(D, V, d, e);
        out.append("Matrix of eigenvalues" + newLine + D);
        out.append("Matrix of eigenvectors" + newLine + V);
        out.append("Real part of eigenvalues" + newLine + d);
        out.append("Imaginary part of eigenvalues" + newLine + e);

        final Matrix Vi = new Matrix(n);
        Vi.invert(V);
        D.mul(D, Vi);
        D.mul(V, D);

        out.append("Verify V*D*inv(V)=S" + newLine + D);
        if(Matrix.checkEigenValue(d) == Matrix.DEFINITE_NEGATIVE)
            out.append("The matrix is DEFINTE NEGATIVE" + newLine);

        out
                .append("----- Computes solution of Lyapunov equation S*X + X*S' = -C:"
                        + newLine);
        final Matrix C = new Matrix(n);
        out.append("With C=" + newLine + C);
        final Matrix X = S.lyap(C);
        out.append("Solution X=" + newLine + X);
        d = new RVector(X.eig());
        if(X.isSymmetric()
                && Matrix.checkEigenValue(d) == Matrix.DEFINITE_POSITIVE)
            out.append("The solution X is SYMMETRIC and DEFINITE POSITIVE"
                    + newLine);

        final Matrix P1 = new Matrix(n);
        final Matrix P2 = new Matrix(n);
        P2.mulTransposeRight(X, S);
        P1.mul(S, X);
        P1.add(P2);
        out.append("Verify S*X + X*S' = -C:" + newLine + P1);
        /*
         * out.append("==============================="); out.append("
         * Rectangular singular matrix");
         * out.append("===============================" + newLine);
         * out.append("R:" + newLine + Rs); m = Rs.getNumRow(); n =
         * Rs.getNumCol();out.append("----- Computes Singular Value
         * Decomposition -----" + newLine); s = new RVector(Math.min(m,n)); rank =
         * Rs.svd(s);out.append("rank: " + rank); out.append("Singular values:" +
         * newLine + s); System.out.printf("Condition number:%9.4f" + newLine +
         * newLine, s.getElement(0)/s.getElement(rank-1));out.append("-----
         * Computes damped least square inverse matrix using SVD decomposition
         * -----" + newLine); Matrix Rsi = new Matrix(n,m); Rsi =
         * Rs.inverse(0.04, 0.0); out.append("inverse (eps=0):" + newLine+ Rsi);
         * Matrix Is = new Matrix(Math.min(m,n)); Is.mul(Rsi,
         * Rs);out.append("computes inv(R)*R" + newLine+Is);
         * 
         * Rsi = Rs.inverse(0.55, 0.04);out.append("inverse (eps=0.55,
         * lambdaMax=0.04):" + newLine+ Rsi);
         * 
         * Is.mul(Rsi, Rs);out.append("computes inv(R)*R" + newLine+Is);
         */
        out.append("===============================" + newLine);
        out.append(" Square quasi singular matrix" + newLine);
        out.append("===============================" + newLine);
        out.append("S:" + newLine + Ss + newLine);
        n = Ss.getNumCol();
        out.append("----- Computes Singular Value Decomposition -----"
                + newLine);
        s = new RVector(n);
        rank = Ss.svd(s);
        out.append("rank: " + rank);
        out.append("Singular values:" + newLine + s);

        out.append(String.format("Condition number:%9.4f" + newLine + newLine,
                s.getElement(0) / s.getElement(rank - 1)));
        out
                .append("----- Computes damped least square inverse matrix using SVD decomposition -----"
                        + newLine);
        Matrix Ssi = new Matrix(n);
        Ssi = Ss.inverse(0.0, 0.0);
        out.append("inverse (eps=0):" + newLine + Ssi);
        final Matrix Is = new Matrix(n);
        Is.mul(Ssi, Ss);
        out.append("computes inv(S)*S" + newLine + Is);

        Ssi = Ss.inverse(0.02, 0.002);
        out.append("inverse (eps=0.02, lambdaMax=0.002):" + newLine + Ssi);

        Is.mul(Ssi, Ss);
        out.append("computes inv(S)*S" + newLine + Is);

        Ssi = Ss.inverse(0.02, 0.02);
        out.append("inverse (eps=0.02, lambdaMax=0.02):" + newLine + Ssi);

        Is.mul(Ssi, Ss);
        out.append("computes inv(S)*S" + newLine + Is);

        Ssi = Ss.inverse(0.02, 0.2);
        out.append("inverse (eps=0.02, lambdaMax=0.2):" + newLine + Ssi);

        Is.mul(Ssi, Ss);
        out.append("computes inv(S)*S" + newLine + Is);
    }

}
