/*
 * TestRNE.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package example;

import java.awt.Dimension;
import java.io.IOException;
import java.net.URISyntaxException;

import javax.robotics.engine.robots.DHRobot;
import javax.robotics.engine.robots.Robot;
import javax.robotics.engine.trajectory.Trajectory;
import javax.robotics.engine.trajectory.Trapezoidal;
import javax.robotics.ode.Solver;
import javax.robotics.plot.MultiPlot;
import javax.robotics.plot.MultiPlotFrame;
import javax.robotics.vecmath.Matrix;
import javax.robotics.vecmath.RMatrix3d;
import javax.robotics.vecmath.RVector3d;
import javax.robotics.vecmath.RVector;
import javax.swing.JOptionPane;

import org.apache.xmlbeans.XmlException;

/**
 * Solution of problem 4.6 of Modelling and Control of Robot Manipulators by
 * Sciavicco, Siciliano(2nd Edition)
 * 
 * @author Carmine Lia
 * 
 */
public class TestRNE
{
    /**
     * 
     */
    private static final long serialVersionUID = 3257568412476584242L;
    private final ClassLoader cl = this.getClass().getClassLoader();
    
    MultiPlotFrame plotsWindow;
    MultiPlot plotPos, plotVel, plotAcc, plotTorque, plotInertia,
            plotCentCoriol, plotGravity, plotTorqueExtForce;
    double dur;
    RVector q, qp, qpp, tau;
    RVector tau_inertia, tau_f, tau_corcen, tau_gravity, h_;
    RVector3d fext, next, h;
    static Integer typeTrj;
    Robot robot;
    Matrix inertia, J, J2x2;
    Solver rk4;
    Trajectory trj;

    public TestRNE() throws XmlException, IOException, URISyntaxException
    {

        J2x2 = new Matrix(2);
        dur = 0.60;
        double dt = 0.001;

        q = new RVector(2);
        qp = new RVector(2);
        qpp = new RVector(2);

        tau = new RVector(2);
        tau_f = new RVector(2);
        tau_inertia = new RVector(2);
        tau_corcen = new RVector(2);
        tau_gravity = new RVector(2);

        // end eff applied force
        // base frame
        h = new RVector3d(500.0, 500.0, 0.0);
        h_ = new RVector(500.0, 500.0);
        // end eff frame
        fext = new RVector3d();
        next = new RVector3d();
        
        String[] possibleValues =
            { "same duration", "different duration" };
        String selectedValue = (String) JOptionPane.showInputDialog(null,
                 "Joints trajectory with", "Choose trajectory",
                JOptionPane.INFORMATION_MESSAGE, null, possibleValues,
                possibleValues[0]);
       
        if(selectedValue == null)
            System.exit(0);
        
        if(selectedValue.equals("same duration"))
        {
            trj = new Trapezoidal(cl.getResourceAsStream("example/conf/trapezoidal_path_same_dur.xml"));
            typeTrj = 1;
        }
        else
        {
            trj = new Trapezoidal(cl.getResourceAsStream("example/conf/trapezoidal_path_diff_dur.xml"));
            typeTrj = 2;
        }
       
        createPlotPanel();

        // frame.printAll(null);

        // init robot setting the joint position at time 0
        robot = new DHRobot(cl.getResource("example/conf/rr_book_chap4.xml").toURI());
        robot.setGravity(new RVector3d(0.0, 9.81, 0.0));
        System.out.println(robot.getGravity());
        final RMatrix3d R = new RMatrix3d();

        for (double t = 0.0; t <= dur; t += dt)
        {
            trj.interpolate(t, q, qp, qpp);

            robot.setQ(q);
            robot.kine().get(R);
            R.transpose();
            fext.mul(R, h);
            fext.negate();
            tau = robot.torque(q, qp, qpp, fext, next);
            inertia = robot.inertia(q);
            tau_inertia.mul(inertia, qpp);
            tau_corcen = robot.torqueC(qp);
            tau_gravity = robot.torqueGravity();
            robot.jacobian().getSubMatrix(0, 1, 0, 1, J2x2);

            J2x2.transpose();
            tau_f.mul(J2x2, h_);

            // tau.add(tau_f);

            updatePlots(t);
        }
        fillPlots();
        // System.out.println("T\n" + robot.kine());
    }
    private void createPlotPanel()
    {

        plotsWindow = (typeTrj == 1) ? new MultiPlotFrame(
                "Dynamics of 2 rev links planar robot (1st trajectory of Example 4.2)",
                4, 2)
                : new MultiPlotFrame(
                        "Dynamics of 2 rev links planar robot (2nd trajectory of Example 4.2)",
                        4, 2);

        plotPos = new MultiPlot("Position", "[s]", "[rad]", "Joint 1",
                "Joint 2");
        plotPos.setXRange(0.0, dur);
        plotVel = new MultiPlot("Velocity", "[s]", "[rad/s]", "Joint 1",
                "Joint 2");
        plotVel.setXRange(0.0, dur);
        plotAcc = new MultiPlot("Acceleration", "[s]", "[rad/s^2]", "Joint 1",
                "Joint 2");
        plotAcc.setXRange(0.0, dur);
        plotTorque = new MultiPlot("Torque", "[s]", "[Nm]", "Joint 1",
                "Joint 2");
        plotTorque.setXRange(0.0, dur);

        plotInertia = new MultiPlot("Inertia torque", "[s]", "[Nm]", "Joint 1",
                "Joint 2");
        plotInertia.setXRange(0.0, dur);

        plotCentCoriol = new MultiPlot("Coriolis+Centrifug torque", "[s]",
                "[Nm]", "Joint 1", "Joint 2");
        plotCentCoriol.setXRange(0.0, dur);
        plotGravity = new MultiPlot("Gravity torque", "[s]", "[Nm]", "Joint 1",
                "Joint 2");
        plotGravity.setXRange(0.0, dur);
        plotTorqueExtForce = new MultiPlot("External force torque", "[s]",
                "[Nm]", "Joint 1", "Joint 2");
        plotTorqueExtForce.setXRange(0.0, dur);

        plotsWindow.setPreferredSize(new Dimension(1024, 768));
        plotsWindow.add(plotPos);
        plotsWindow.add(plotVel);
        plotsWindow.add(plotAcc);
        plotsWindow.add(plotTorque);
        plotsWindow.add(plotInertia);
        plotsWindow.add(plotCentCoriol);
        plotsWindow.add(plotGravity);
        plotsWindow.add(plotTorqueExtForce);
        plotsWindow.create();
        plotsWindow.setDefaultCloseOperation(MultiPlotFrame.EXIT_ON_CLOSE);
        return;
    }
    private void fillPlots()
    {

        plotPos.fillPlot();
        plotVel.fillPlot();
        plotAcc.fillPlot();
        plotTorque.fillPlot();
        plotInertia.fillPlot();
        plotCentCoriol.fillPlot();
        plotGravity.fillPlot();
        plotTorqueExtForce.fillPlot();
    }

    private void updatePlots(double t)
    {
        plotPos.addVector(t, q, true);
        plotVel.addVector(t, qp, true);
        plotAcc.addVector(t, qpp, true);
        plotTorque.addVector(t, tau, true);
        plotInertia.addVector(t, tau_inertia, true);
        plotCentCoriol.addVector(t, tau_corcen, true);
        plotGravity.addVector(t, tau_gravity, true);
        plotTorqueExtForce.addVector(t, tau_f, true);
    }
    /**
     * @param args
     * @throws IOException
     * @throws XmlException
     * @throws URISyntaxException 
     */
    public static void main(String[] args) throws XmlException, IOException, URISyntaxException
    {
        new TestRNE();
    }

}
