/*
 * ConstraintVelocity.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.engine.clik;

import javax.robotics.vecmath.RVector;

/**
 * Interface for computing the constraint velocity.<br>
 * <img src="doc-files/vincolovel.png" alt="q0'=k0*grad(w(q))" />
 * 
 * @author Carmine Lia
 * @since 1.0.2
 */
public interface ConstraintVelocity
{
    /**
     * Gets the velocity constraint gain.
     * 
     * @return the gain value.
     */
    public double getK0();
    /**
     * Sets the velocity constraint gain.
     * 
     * @param k0
     *            the gain value to set.
     */
    public void setK0(double k0);

    /**
     * Computes the constraint velocity q0'.<br>
     * <img src="doc-files/vincolovel.png" alt="q0'=k0*grad(w(q))" />
     * 
     * @param q
     *            the joints position.
     * @return the current value of constraint.
     */
    public RVector compute(RVector q);

}
