package javax.robotics.image;

/*
 *	@(#)ImagePrinter.java 1.5 02/04/01 15:04:11
 *
 * Copyright (c) 1996-2002 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistribution in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *
 * Neither the name of Sun Microsystems, Inc. or the names of
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * This software is provided "AS IS," without a warranty of any
 * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
 * WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
 * EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN
 * OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR
 * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE SOFTWARE,
 * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * You acknowledge that Software is not designed,licensed or intended
 * for use in the design, construction, operation or maintenance of
 * any nuclear facility.
 */

import java.awt.print.*;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;
import java.awt.geom.AffineTransform;

/**
 * Utility class to print images.
 */
final class ImagePrinter implements Printable, ImageObserver
{
    BufferedImage bImage;

    public int print(Graphics g, PageFormat pf, int pi) throws PrinterException
    {

        if(pi >= 1)
        {
            return Printable.NO_SUCH_PAGE;
        }
        // g2d.translate(pf.getImageableX(), pf.getImageableY());
        final AffineTransform t2d = new AffineTransform();
        t2d.translate(pf.getImageableX(), pf.getImageableY());
        final double xscale = pf.getImageableWidth() / (double) bImage.getWidth();
        final double yscale = pf.getImageableHeight() / (double) bImage.getHeight();
        final double scale = Math.min(xscale, yscale);
        t2d.scale(scale, scale);
        try
        {
            final Graphics2D g2d = (Graphics2D) g;
            g2d.drawImage(bImage, t2d, this);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            return Printable.NO_SUCH_PAGE;
        }
        return Printable.PAGE_EXISTS;
    }

    final void print()
    {
        final PrinterJob printJob = PrinterJob.getPrinterJob();
        PageFormat pageFormat = printJob.defaultPage();
    //    pageFormat.setOrientation(PageFormat.LANDSCAPE);
        pageFormat = printJob.validatePage(pageFormat);
        printJob.setPrintable(this, pageFormat);
        if(printJob.printDialog())
        {
            try
            {
                printJob.print();
            }
            catch (PrinterException ex)
            {
                ex.printStackTrace();
            }
        }
    }

    public boolean imageUpdate(Image img, int infoflags, int x, int y,
            int width, int height)
    {
        return false;
    }

    ImagePrinter(BufferedImage bImage)
    {
        this.bImage = bImage;
    }
}
