package javax.robotics.j3d;

/*
 *      Axis.java 1.0 98/11/25
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import javax.media.j3d.*;
import javax.vecmath.*;


/**
 * Java3D utility class to draw the axes of cartesian space.
 * <p>the X axis is red.</p>
 * <p>the Y axis is green.</p>
 * <p>the Z axis is blue.</p>
 */
public class Axis extends Shape3D
{
	private float a, b, c;

	////////////////////////////////////////////
	//
	// create axis visual object
	//
	/**
	 * Default constructor.
	 */
	public Axis()
	{
        this(1.0f);
	
	}
	/**
     * Constructor with axes length.
	 * @param a the axes length.
	 */
	public Axis(float a)
	{
		if ( a > 0.1f )
		{
			this.a = a;
			b= a/10.0f;
			c = a-b;
		}
		else
		{
			this.a = 1.0f;
			b= 0.1f;
			c= 0.9f;
		}
		this.setGeometry(createGeometry());
 	}

	private Geometry createGeometry()
	{

		final LineArray axisLines = new LineArray(18, LineArray.COORDINATES | LineArray.COLOR_3);
		// create line for X axis
		axisLines.setCoordinate(0, new Point3f(0.0f, 0.0f, 0.0f));
		axisLines.setCoordinate(1, new Point3f(a, 0.0f, 0.0f));
		axisLines.setCoordinate(2, new Point3f(c, 0.0f, b));
		axisLines.setCoordinate(3, new Point3f(a, 0.0f, 0.0f));
		axisLines.setCoordinate(4, new Point3f(c, 0.0f, -b));
		axisLines.setCoordinate(5, new Point3f(a, 0.0f, 0.0f));
		
		// create line for Y axis
		axisLines.setCoordinate(6, new Point3f(0.0f, 0.0f, 0.0f));
		axisLines.setCoordinate(7, new Point3f(0.0f, a, 0.0f));
		axisLines.setCoordinate(8, new Point3f(b, c, 0.0f));
		axisLines.setCoordinate(9, new Point3f(0.0f, a, 0.0f));
		axisLines.setCoordinate(10, new Point3f(-b, c, 0.0f));
		axisLines.setCoordinate(11, new Point3f(0.0f, a, 0.0f));
		// create line for Z axis
		axisLines.setCoordinate(12, new Point3f(0.0f, 0.0f, 0.0f));
		axisLines.setCoordinate(13, new Point3f(0.0f, 0.0f, a));
		axisLines.setCoordinate(14, new Point3f(b, 0.0f, c));
		axisLines.setCoordinate(15, new Point3f(0.0f, 0.0f, a));
		axisLines.setCoordinate(16, new Point3f(-b, 0.0f, c));
		axisLines.setCoordinate(17, new Point3f(0.0f, 0.0f, a));
		
		//asse X rosso
		axisLines.setColor(0, ColorConstants.RED);
		axisLines.setColor(1, ColorConstants.RED);
		axisLines.setColor(2, ColorConstants.RED);
		axisLines.setColor(3, ColorConstants.RED);
		axisLines.setColor(4, ColorConstants.RED);
		axisLines.setColor(5, ColorConstants.RED);
		
		//asse Y verde
		axisLines.setColor(6, ColorConstants.GREEN);
		axisLines.setColor(7, ColorConstants.GREEN);
		axisLines.setColor(8, ColorConstants.GREEN);
		axisLines.setColor(9, ColorConstants.GREEN);
		axisLines.setColor(10, ColorConstants.GREEN);
		axisLines.setColor(11, ColorConstants.GREEN);
		
		//asse Z blu
		axisLines.setColor(12, ColorConstants.BLUE);
		axisLines.setColor(13, ColorConstants.BLUE);
		axisLines.setColor(14, ColorConstants.BLUE);
		axisLines.setColor(15, ColorConstants.BLUE);
		axisLines.setColor(16, ColorConstants.BLUE);
		axisLines.setColor(17, ColorConstants.BLUE);
   
		return axisLines;

	} // end of Axis createGeometry()


} // end of class Axis
