/*
 * Lights.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.j3d;



import javax.media.j3d.AmbientLight;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.DirectionalLight;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3f;

/**
 * This class creates a system of lights to add a 3D scene. 
 * Composes an ambient light and three directional lights.
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.0.3
 * @version 4/11/2005
 */
public class Lights extends BranchGroup
{
	/**
	 * Composes an ambient light and three directional lights.
	 */
	public Lights()
	{
		super();
		//Impostazione della sfera di azione per le luci
		BoundingSphere lightBounds = new BoundingSphere(new Point3d(), 1000.0);
		// Impostazione della luce di ambiente
		Color3f ambientColor = new Color3f(1.0f, .80f, .70f);
		AmbientLight ambientLightNode = new AmbientLight(ambientColor);
		ambientLightNode.setInfluencingBounds(lightBounds);
		addChild(ambientLightNode);

		// Impostazione delle luci direzionali
		Color3f light1Color = new Color3f(1.0f, 1.0f, 0.7f);
		Vector3f light1Direction  = new Vector3f(1.0f, 1.0f, 5.0f);
		
		Color3f light2Color = new Color3f(1.0f, 1.0f, .70f);
		Vector3f light2Direction  = new Vector3f(-1.0f, -1.0f, -5.0f);
		
		Color3f light3Color = new Color3f(.60f, .60f, .40f);
		Vector3f light3Direction  = new Vector3f(-1.0f, 1.0f, -1.0f);


		DirectionalLight light1
		= new DirectionalLight(light1Color, light1Direction);
		light1.setInfluencingBounds(lightBounds);
		addChild(light1);

		DirectionalLight light2
		= new DirectionalLight(light2Color, light2Direction);
		light2.setInfluencingBounds(lightBounds);
		addChild(light2);

		DirectionalLight light3
		= new DirectionalLight(light3Color, light3Direction);
		light3.setInfluencingBounds(lightBounds);
		addChild(light3);

	}
}

