/*
 * RobotCanvas3D.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.j3d;

import java.awt.Dimension;
import java.awt.GraphicsConfiguration;
import java.awt.Point;

import javax.media.j3d.Canvas3D;
import javax.media.j3d.Screen3D;

import com.sun.j3d.utils.universe.SimpleUniverse;
/**
 * Class to create a Java3D scene to display the robot.
 * It's available also the off screen Canvas3D.
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.2
 * @version 04/11/2005
 */
public class RobotCanvas3D extends Canvas3D
{

    /**
     * 
     */
    private static final long serialVersionUID = 3575063212140139703L;
   
    
    private OffScreenCanvas3D offScreenCanvas3D;
    private GraphicsConfiguration gc;
    
    /**
     * Default constructor.
     */
    public RobotCanvas3D()
    {
        this(SimpleUniverse.getPreferredConfiguration());
    }
    
    /**
     * Contructor with graphic configuration.
     * @param gc the graphic configuration
     */
    public RobotCanvas3D(GraphicsConfiguration gc)
    {
        super(gc);
        this.gc = gc;
    }

    /**
     * Gets the off screen canvas so that it can be rendered as an image.
     * @return the off screen canvas.
     */
    public OffScreenCanvas3D getOffScreenCanvas3D()
    {
        offScreenCanvas3D = new OffScreenCanvas3D(gc);
        final Point loc = getLocationOnScreen();
        final Screen3D sOn = getScreen3D();
        final Screen3D sOff = offScreenCanvas3D.getScreen3D();
        Dimension dim = sOn.getSize();
        sOff.setSize(dim);
        sOff.setPhysicalScreenWidth(sOn.getPhysicalScreenWidth());
        sOff.setPhysicalScreenHeight(sOn.getPhysicalScreenHeight());
        offScreenCanvas3D.setOffScreenLocation(loc);
        return offScreenCanvas3D;
    }
    
   
}
