/*
 * EndEffPosControlSimulation.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.simulator;

import java.util.Enumeration;

import javax.robotics.engine.controllers.EndEffController;
import javax.robotics.engine.robots.Robot3D;
import javax.robotics.engine.trajectory.Trajectory;
import javax.robotics.ode.Solver2;

/**
 * 
 * @author carmine lia
 * @version 01/nov/06
 *
 */
public class EndEffPosControlSimulation extends EndEffControlSimulation
{
    
    public EndEffPosControlSimulation(Robot3D robot, EndEffController controller, Solver2 solver, Trajectory trj, int points)
    {
        super(robot, controller, solver, trj, points);
    }

    @Override
    public void initialize()
    {
        time = solver.getTime();
        q.set(solver.getLastValuePosition());
        qp.set(solver.getLastValueVelocity());
        T.set(robot.kine());
        T.get(p);
        q.get(jointData[i]);
        robot.setQ3D(q);

        // end effector path
        trj.interpolate(time, s, ds, dds);
     
        // 
        tau.set(controller.torqueCmd(dds, ds, s));
        solver.step(q, qp);

        wakeupOn(wakeUp);

        sceneTimer.scheduleAtFixedRate(new Update3DScene(), 0l,
                updateSchedulerTime);

        createPlotPanel();
        updatePlotPanel();

    }

    @Override
    public void processStimulus(Enumeration arg0)
    {
        if(i >= points - 1)
        {
            this.setEnable(false);
            sceneTimer.cancel();
            plotPanel.fillPlots();
            return;
        }

        time = solver.getTime();
        q.set(solver.getLastValuePosition());
        qp.set(solver.getLastValueVelocity());
        T.set(robot.kine());
        T.get(p);
        
        i++;
        q.get(jointData[i]);
        
        wakeupOn(wakeUp);

        // end effector path
        trj.interpolate(time, s, ds, dds);
        
        // compute joints torque
        tau.set(controller.torqueCmd(dds, ds, s));
        solver.step(q, qp);

        updatePlotPanel();
    }

   
    private final void createPlotPanel()
    {

        plotPanel.add(plotJPos);
        plotPanel.add(plotJVel);
        plotPanel.add(plotJTorque);
        plotPanel.add(plotEndEffErrPos);
        return;
    }

    private final void updatePlotPanel()
    {
        // plotting joints pos
        plotJPos.addVector(time, q, true);
        // plotting joints vel
        plotJVel.addVector(time, qp, true);
//      plotting joints torque
        plotJTorque.addVector(time, tau, true);
        // plot end effector err pos/orien
        p.sub(s);
        plotEndEffErrPos.addPoint(0, time, p.length(), true);
    }

    
}
