/*
 * DynamicsPanel.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.swing;


import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.robotics.engine.robots.Robot3D;
import javax.robotics.vecmath.RVector3d;
import javax.swing.BorderFactory;
import javax.swing.JPanel;

/**
 * This class creates a panel with spinner for dynamics.
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.1
 * @version 15/10/2005
 * 
 */
public class DynamicsPanel extends JPanel
{
    ExternalForcePanel extForce;
    GravityPanel gravPanel;
    /**
     * 
     */
    private static final long serialVersionUID = 3257572823407867188L;

   
    /**
     * Constructor of the panel.
     * @param robot the robot.
     * @param Fext the external force.
     * @param Next the external torque.
     */
    public DynamicsPanel(Robot3D robot, RVector3d Fext, RVector3d Next)
    {
        super();
        gravPanel =  new GravityPanel(robot);
        ExternalForcePanel extForce = new ExternalForcePanel(Fext, Next);
        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.VERTICAL;
        setLayout(gbl);
        setBorder(BorderFactory.createCompoundBorder(BorderFactory
                .createTitledBorder(BorderFactory.createLineBorder(Color.gray),
                        "Dynamics"), BorderFactory.createEmptyBorder(2, 5, 2,
                5)));
        add(gravPanel, c);
        c.gridy = 1;
        add(extForce, c);
    }
    
    /**
     * Gets the gravity panel.
     * @return the gravity panel.
     * @see GravityPanel
     */
    public final GravityPanel getGravityPanel()
    {
        return gravPanel;
    }
    
    /**
     * Gets the external force panel.
     * @return the external force panel.
     * @see ExternalForcePanel
     */
    public final ExternalForcePanel getExternalForcePanel()
    {
        return extForce;
    }
}
