/*
 * ExternalForcePanel.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.swing;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.robotics.vecmath.RVector3d;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * This class creates a panel of spinner for external forces an torques.
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.0
 */
public class ExternalForcePanel extends JPanel implements ChangeListener
{

    /**
     * 
     */
    private static final long serialVersionUID = 3978143257605911607L;

    private JSpinner fextx, fexty, fextz, nextx, nexty, nextz;
    private RVector3d Fext, Next;

    /**
     * Creates the panel of spinners for input forces and torques.
     */
    public ExternalForcePanel(RVector3d Fext, RVector3d Next)
    {
        super();
        this.Fext = Fext;
        this.Next = Next;
        createPanel();
        setValues();
    }

    private void setValues()
    {

        fextx.setValue(Fext.x);
        fexty.setValue(Fext.y);
        fextz.setValue(Fext.z);

        nextx.setValue(Next.x);
        nexty.setValue(Next.y);
        nextz.setValue(Next.z);

    }
    
    final void resetValues()
    {
        Fext.set(0.0,0.0,0.0);
        Next.set(0.0,0.0,0.0);
        setValues();
    }
    
    private JSpinner createJSpinner(String tip)
    {
        final JSpinner spinner = new JSpinner(new SpinnerNumberModel(0.0, -5000.0, 5000.0, 1.0));
        spinner.addChangeListener(this);
        spinner.setToolTipText(tip);
  //      System.out.println(spinner.getPreferredSize());
        spinner.setPreferredSize(new Dimension(60, 20));
        return spinner;
    }
    
    private JPanel createForcePanel()
    {
        // force panel
        final JPanel fp = new JPanel();
        fp.setLayout(new GridLayout(3, 2, 0, 0));

        fextx = createJSpinner("force x");
        fp.add(fextx);

        fexty = createJSpinner("force y");
        fp.add(fexty);

        fextz = createJSpinner("force z");
        fp.add(fextz);
        return fp;
    }

    private JPanel createTorquePanel()
    {
        // torque panel
        final JPanel tp = new JPanel();
        tp.setLayout(new GridLayout(3, 2, 0, 0));
       
        nextx = createJSpinner("torque x");
        tp.add(nextx);

        nexty = createJSpinner("torque y");
        tp.add(nexty);

        nextz = createJSpinner("torque z");
        tp.add(nextz);
        return tp;
    }
    
    private void createPanel()
    {
        // create the main panel
        final BorderLayout layout = new BorderLayout();
        layout.setHgap(2);
        layout.setVgap(5);
        setLayout(layout);
        
        setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(BorderFactory
                .createLineBorder(Color.gray), "External Force"),BorderFactory.createEmptyBorder(2,5,2,5)));
       
        
        add(createForcePanel(),BorderLayout.WEST);
        add(createTorquePanel(),BorderLayout.EAST);
        
        final JButton reset = new JButton("Reset");
        reset.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               resetValues();
            }
        });
        add(reset, BorderLayout.SOUTH);
    }

    private void getFext()
    {
        Fext.x = (Double) fextx.getValue();
        Fext.y = (Double) fexty.getValue();
        Fext.z = (Double) fextz.getValue();
    }

    private void getNext()
    {
        Next.x = (Double) nextx.getValue();
        Next.y = (Double) nexty.getValue();
        Next.z = (Double) nextz.getValue();

    }

    public void stateChanged(ChangeEvent e)
    {
        getFext();
        getNext();
    }

}
