/*
 * RemoteFile.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;


/**
 * Utility class to select a remote file.
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.2
 * @version 04/11/2005
 */
public class RemoteFile
{
    private final File outputFile;
    
    /**
     * Constructor from the url string.
     * @param xmlURL the string url of the remote file.
     * @throws MalformedURLException
     * @throws IOException
     */
    public RemoteFile(String xmlURL) throws MalformedURLException, IOException
    {
        this(new URL(xmlURL));
    }
    
    /**
     * Constructor from the url.
     * @param xmlURL the URL of the remote file.
     * @throws IOException
     */
    public RemoteFile(URL xmlURL) throws IOException
    {
        final URLConnection yc;
        String inputLine;
        
        yc = xmlURL.openConnection();

        final BufferedReader in = new BufferedReader(new InputStreamReader(yc
                .getInputStream()));
        outputFile = File.createTempFile("tmp", "");
        final FileWriter out = new FileWriter(outputFile);
        while ((inputLine = in.readLine()) != null)
            out.write(inputLine);

        in.close();
        out.close();
        
    }

    /**
     * Gets the remote file.
     * @return the file.
     */
    final public File getFile()
    {
        return outputFile;
    }
}
